@@ -26,11 +26,12 @@ from pathlib import Path
 from typing import Generator, List, Optional, Tuple
 
 from ._util import no_echo, removesuffix, resolve_relative, run
+from .appconfig import AppConfig
 from .clone import Clone, map_paths
 from .const import MANIFEST_PATH_DEFAULT, MANIFESTS_PATH
 from .datamodel import GroupFilters, Manifest, ManifestSpec, Project, ProjectPaths, ProjectSpec
 from .deptree import DepNode, get_deptree
-from .exceptions import GitCloneNotCleanError, InitializedError, ManifestExistError
+from .exceptions import GitCloneNotCleanError, GitTagExistsError, InitializedError, ManifestExistError
 from .git import DiffStat, Git, Status
 from .iters import ManifestIter, ProjectIter
 from .manifestfinder import find_manifest
@@ -257,7 +258,8 @@ class GitWS:
             Workspace.check_empty(path, main_path)
         secho(f"===== {resolve_relative(main_path)} (MAIN {name!r}) =====", fg=_COLOR_BANNER)
         secho(f"Cloning {url!r}.", fg=_COLOR_ACTION)
-        git = Git(resolve_relative(main_path), secho=secho)
+        clone_cache = AppConfig().options.clone_cache
+        git = Git(resolve_relative(main_path), clone_cache=clone_cache, secho=secho)
         git.clone(url, revision=revision)
         return GitWS.create(path, main_path, manifest_path=manifest_path, group_filters=group_filters, secho=secho)
 
@@ -403,7 +405,7 @@ class GitWS:
             for diffstat in clone.git.diffstat(paths=cpaths):
                 yield diffstat.with_path(path)
 
-    def checkout(self, paths: Tuple[Path, ...], force: bool = False):
+    def checkout(self, paths: Optional[Tuple[Path, ...]] = None, force: bool = False):
         """
         Enriched Git Checkout.
 
@@ -507,7 +509,7 @@ class GitWS:
                 clone.check()
                 clone.git.commit(msg, all_=all_)
 
-    def tag(self, name: str, msg: Optional[str] = None):
+    def tag(self, name: str, msg: Optional[str] = None, force: bool = False):
         """
         Create Git Tag.
 
@@ -520,16 +522,22 @@ class GitWS:
         clone = Clone.from_project(self.workspace, next(self.projects()), secho=self.secho)
         self.secho(f"===== {clone.info} =====", fg=_COLOR_BANNER)
         git = clone.git
+        # check
+        if not force and git.get_tags(name):
+            raise GitTagExistsError(name)
         # freeze
         manifest_path = MANIFESTS_PATH / f"{name}.toml"
         manifest_spec = self.get_manifest_spec(freeze=True, resolve=True)
         (self.main_path / MANIFESTS_PATH).mkdir(exist_ok=True, parents=True)
+        (self.main_path / manifest_path).touch()
         manifest_spec.save(self.main_path / manifest_path)
         # commit
-        git.add((manifest_path,), force=True)
-        git.commit(msg or name, paths=(manifest_path,))
+        paths = (manifest_path,)
+        git.add(paths, force=True)
+        if any(git.status(paths=paths)):
+            git.commit(msg or name, paths=paths)
         # tag
-        git.tag(name, msg=msg)
+        git.tag(name, msg=msg, force=force)
 
     def run_foreach(
         self,
